import { Test, TestingModule } from '@nestjs/testing';
import { AuthController } from './auth.controller';
import { AuthService } from './auth.service';
import { signInDto } from './dto/signIn.dto';
import { Response } from 'express';

describe('AuthController', () => {
  let controller: AuthController;
  let authService: AuthService;

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      controllers: [AuthController],
      providers: [
        {
          provide: AuthService,
          useValue: {
            logIn: jest.fn(),
          },
        },
      ],
    }).compile();

    controller = module.get<AuthController>(AuthController);
    authService = module.get<AuthService>(AuthService);
  });

  it('should be defined', () => {
    expect(controller).toBeDefined();
  });

  describe('logIn', () => {
    it('should return an error if the login is not provided', () => {
      const body = { password: 'password' } as signInDto;
      const response = { json: jest.fn() } as unknown as Response;

      try {
        controller.logIn(body, response);
      } catch (error) {
        expect(error.status).toBe(400);
      }
    });

    it('should return an error if the password is not provided', () => {
      const body = { login: 'login' } as signInDto;
      const response = { json: jest.fn() } as unknown as Response;

      try {
        controller.logIn(body, response);
      } catch (error) {
        expect(error.status).toBe(400);
      }
    });

    it('should call authService.logIn with correct parameters', () => {
      const body = { login: 'login', password: 'password' } as signInDto;
      const response = { json: jest.fn() } as unknown as Response;

      controller.logIn(body, response);

      expect(authService.logIn).toHaveBeenCalledWith(
        body.login,
        body.password,
        response,
      );
    });

    it('should log in the user', () => {
      const body = { login: 'login', password: 'password' } as signInDto;
      const response = { json: jest.fn() } as unknown as Response;

      controller.logIn(body, response);

      expect(authService.logIn).toHaveBeenCalled();
    });
  });
});
