import {
  Body,
  Controller,
  Post,
  HttpCode,
  HttpStatus,
  UsePipes,
  ValidationPipe,
  Res,
  BadRequestException,
} from '@nestjs/common';
import { AuthService } from './auth.service';
import { signInDto } from './dto/signIn.dto';
import { Response } from 'express';
import { ValidationError } from 'class-validator';

@Controller('auth')
export class AuthController {
  constructor(private authService: AuthService) {}

  @HttpCode(HttpStatus.OK)
  @Post('login')
  @UsePipes(
    new ValidationPipe({
      whitelist: true,
      forbidNonWhitelisted: true,
      exceptionFactory: (validationErrors: ValidationError[] = []) => {
        return new BadRequestException(validationErrors);
      },
    }),
  )
  logIn(@Body() signInDto: signInDto, @Res() res: Response) {
    return this.authService.logIn(signInDto.login, signInDto.password, res);
  }

  @HttpCode(HttpStatus.OK)
  @Post('logout')
  logOut(@Res() res: Response) {
    return this.authService.logOut(res);
  }
}
