import { Injectable, Res, UnauthorizedException } from '@nestjs/common';
import { UserService } from '../../domain/user/user.service';
import { JwtService } from '@nestjs/jwt';
import * as bcrypt from 'bcrypt';
import { Response } from 'express';

@Injectable()
export class AuthService {
  constructor(
    private userService: UserService,
    private jwtService: JwtService,
  ) {}

  async logIn(login: string, pass: string, @Res() res: Response): Promise<any> {
    const user = await this.userService.findOneByLogin(login);
    if (!user || !(await bcrypt.compare(pass, user.password))) {
      throw new UnauthorizedException();
    }
    const payload = { id: user.id, username: user.username };

    const accessToken = await this.jwtService.signAsync(payload);

    res.cookie('access_token', accessToken, {
      httpOnly: true,
      secure: true,
      partitioned: true,
      sameSite: 'none',
      maxAge: 24 * 60 * 60 * 1000, // 24 hours
    });

    res.send({
      success: true,
    });
  }

  async logOut(@Res() res: Response): Promise<any> {
    res.clearCookie('access_token');
    res.send({
      success: true,
    });
  }
}
