import { Injectable, CanActivate, ExecutionContext } from '@nestjs/common';
import { JwtService } from '@nestjs/jwt';

@Injectable()
export class OptionalAuthGuard implements CanActivate {
  constructor(private readonly jwtService: JwtService) {}

  async canActivate(context: ExecutionContext): Promise<boolean> {
    const request = context.switchToHttp().getRequest();

    // Récupérer le token dans les cookies
    const token = request.cookies?.['access_token'];

    if (!token) {
      // Aucun token, la requête est autorisée sans authentification
      return true;
    }

    try {
      const user = await this.jwtService.verifyAsync(token);
      request.userId = user.id; // Ajoute `userId` à `req`
    } catch (error) {
      console.warn('Invalid token, proceeding as unauthenticated');
    }

    return true; // Toujours autoriser la requête
  }
}
