import { PrismaService } from '../../../infra/prisma/prisma-service';
import { AnswerRepository } from './answer-repository';
import { PrismaClient } from '@prisma/client';

export class AnswerRepositoryOnDb implements AnswerRepository {
  constructor(private readonly prismax: PrismaService) {}
  private readonly prisma = new PrismaClient();

  async isCorrect(
    codeQuiz: string,
    questionID: number,
    answerID: number,
  ): Promise<boolean> {
    const answer = await this.prisma.answer.findUnique({
      where: {
        id: answerID,
      },
    });

    if (!answer) {
      throw new Error('Answer not found for the specified question or quiz.');
    }

    // Retourne le booléen `isCorrect`
    return answer.isCorrect;
  }

  async isRunCorrect(
    id: string,
    questionID: number,
    answerID: number,
  ): Promise<boolean> {
    const answer = await this.prisma.answer.findUnique({
      where: {
        id: answerID,
      },
    });

    if (!answer) {
      throw new Error('Answer not found for the specified question or quiz.');
    }

    await this.prisma.answerRun.create({
      data: {
        runId: id,
        questionId: questionID,
        answerId: answerID,
      },
    });

    // Retourne le booléen `isCorrect`
    return answer.isCorrect;
  }
}
