import { Injectable } from '@nestjs/common';
import { Mode } from '../../utils/Mode';

@Injectable()
export class OllamaService {

    async generateTrueAnswer(question: string) {
        try {
            const prompt = 'I am creating a quiz with diverse themes and category. I want the true answer of this question the shortest way possible :' + question;
            const body = {
                "model": "llama3.2-vision:11b",
                "prompt": question,
                "stream": false,
                "format": {
                    type: 'object',
                    properties: {
                        answer: {
                            type: 'string'
                        },
                    },
                    required: ['answer']
                },
                num_predict: 128,
                repeat_penalty: 0,
            }

            const response = await fetch(process.env.OLLAMA_URL + 'api/generate', {
                method: 'POST',
                body: JSON.stringify(body)
            });

            const data = await response.json();

            return data.response;
        } catch (error) {
            console.error('Error in generateTrueAnswer:', error.message, error.stack);
            throw new Error('Failed to generate a true answer.');
        }
    }

    async generateFalseAnswer(question: string, mode: Mode) {
        try {
            let prompt;
            switch (mode) {
                case Mode.HUMOUR:
                    prompt = `
You are helping to create a quiz game. 
Your task is to provide a FUNNY and WRONG answer to this question: "${question}". 
- The answer must be short (maximum 10 words).
- The answer must be obviously incorrect and absurd.
- Example: For "Who won the 1998 World Cup?", reply "A team of penguins in bow ties."`;
                    break;

                case Mode.REALISTIC:
                    prompt = `
You are helping to create a quiz game. 
Your task is to provide a REALISTIC but WRONG answer to this question: "${question}". 
- The answer must be short (1 to 3 words).
- The answer must be clearly incorrect.
- Do NOT provide anything that could be interpreted as correct.
- Example: For "What is 2+2?", reply "5."`;
                    break;

                default:
                    throw new Error('Invalid mode');
            }

            const body = {
                model: "llama3.2-vision:11b",
                prompt: prompt,
                stream: false,
                format: {
                    type: 'object',
                    properties: {
                        answer: {
                            type: 'string'
                        },
                    },
                    required: ['answer']
                },
                num_predict: 6, // Réduit encore plus la longueur maximale
                repeat_penalty: 1.25, // Encourage des réponses uniques
                temperature: 1.2, // Accentue la créativité pour des réponses absurdes
                top_p: 0.8, // Réduit les options les plus probables pour éviter des réponses classiques
            };


            const response = await fetch(process.env.OLLAMA_URL + 'api/generate', {
                method: 'POST',
                body: JSON.stringify(body)
            });

            const data = await response.json();

            return data.response;
        } catch (error) {
            console.error('Error in generateFalseAnswer:', error.message, error.stack);
            throw new Error('Failed to generate a false answer.');
        }
    }
}
