import {
  IsString,
  IsArray,
  ValidateNested,
  IsOptional,
  IsBoolean, IsBase64
} from "class-validator";
import { Type } from 'class-transformer';
import { IsStringOrBase64 } from "./class-validator";

class CreateCommunityQuizDto {
  @IsString()
  name: string;

  @IsString()
  description: string;

  @IsString()
  difficulty: string;

  @IsArray()
  @ValidateNested({ each: true })
  @Type(() => QuestionDto)
  questions: QuestionDto[];
}

class QuestionDto {
  @IsString()
  text: string;

  @IsBoolean()
  isMultipleChoice: boolean;

  @IsArray()
  @ValidateNested({ each: true })
  @Type(() => AnswerDto)
  answers: AnswerDto[];
}

class AnswerDto {
  @IsStringOrBase64({ message: "text must be a valid string or Base64 encoded string" })
  text: string;

  @IsBoolean()
  isCorrect?: boolean;

  @IsString()
  fileType: string;
}

export { CreateCommunityQuizDto, QuestionDto, AnswerDto };
