import { Module, forwardRef } from '@nestjs/common';
import { RunService } from './run.service';
import { RunController } from './run.controller';
import { PrismaModule } from 'src/infra/prisma/prisma.module';
import { RunRepository } from './store/run-repository';
import { RunRepositoryOnDb } from './store/run-repository-on-db';
import { AnswerModule } from '../answer/answer.module';
import { QuizModule } from '../quiz/quiz.module';
import { UserModule } from '../user/user.module';

@Module({
  imports: [
    PrismaModule,
    AnswerModule,
    forwardRef(() => QuizModule), // Utilisation de forwardRef pour éviter les dépendances circulaires
    forwardRef(() => UserModule),
  ],
  controllers: [RunController],
  providers: [
    RunService,
    {
      provide: RunRepository,
      useClass: RunRepositoryOnDb, // Injection de la classe de repository
    },
  ],
  exports: [
    RunService, // Exporter si nécessaire dans d'autres modules
    RunRepository, // Exporter si un autre module en a besoin
  ],
})
export class RunModule {}
